#################################################################################
# Created       : Martin Foster
# Modified      : 01/10/2001
#################################################################################
#
# Database - Modules part of Webchat designed to aid Database programming  
# Copyright (C) 2000  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2E 0K9
#
#################################################################################

package Webchat::Database;					# Package name

use DBI;							# Database indepedent

#################################################################################
# Data Members
#################################################################################

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Database Connect
#
# Sub-routine to allow connection to the database while revealing as little as
# possible to the coder.  Also centralized

sub DatabaseConnect						# Database reference handle
{
	#####################
	# Data members
	my $Database = shift;					# Database handle

	# User variables
	my $db		= 'webchat';				# Database
	my $server	= 'localhost';				# Server to connect
	my $port	= '3306';				# Port to use
	my $login	= 'webchat';				# Login name
	my $pass	= 'CHANGE';				# Password
	

	#####################
	# Program area

	# Establish connection
	${$Database} = DBI->connect("DBI:mysql:$db:$server:$port",
				$login,
				$pass,
				{PrintError => 1})
	|| die ("Database error \#$DBI::err :  $DBI::errstr");

# PgSql	${$Database} = DBI->connect("DBI:Pg:dbname=$Connect{'SetDatabase'}", 
#		$Connect{'SetUser'},
#		$Connect{'SetPass'})
#	|| die ("Database error \#$DBI::err :  $DBI::errstr");
}


#####################
# Document Get
#
# Data retreival sub-routine for WebchatText table in Webchat database

sub DocumentGet							# Database handle, field
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# SQL statement

	my $field = shift;					# Search field
	my $data;						# Data pulled from database


	#####################
	# Program area

	# Initialize and execute
	$Statement = $Database->prepare("SELECT WebchatTextContent FROM WebchatText WHERE WebchatTextName=?");
	$Statement->execute($field);

	# Retreive and finish
	$data = $Statement->fetchrow();
	$Statement->finish;

	# Return gathered information
	return $data;
}
 
# Alias for Acceptable Use Policy
sub DocumentGetAUP
{ print DocumentGet(shift, 'AcceptableUsePolicy'); }

# Alias for Header
sub DocumentGetHeader
{ print DocumentGet(shift, 'WebHeader'); }

# Alias for Header
sub DocumentGetHeaderFramed
{ print DocumentGet(shift, 'WebHeaderFramed'); }

# Alias for Header
sub DocumentGetHeaderJava
{ print DocumentGet(shift, 'WebHeaderJava'); }

# Alias for Header
sub DocumentGetHeaderPlain
{ print DocumentGet(shift, 'WebHeaderPlain'); }

# Alias for Footer
sub DocumentGetFooter
{ print DocumentGet(shift, 'WebFooter'); }

# Alias for Invitation
sub DocumentGetInvitation
{ return DocumentGet(shift, 'Invitation'); }

# Alias for Sidebar
sub DocumentGetSidebar
{ print DocumentGet(shift, 'WebSidebar'); }

# Alias for Realm
sub DocumentGetRealm
{ print DocumentGet(shift, 'InfoRealm'); }

# Alias for Bio
sub DocumentGetBio
{ print DocumentGet(shift, 'InfoBio'); }

# Alias for Registration
sub DocumentGetRegistration
{ print DocumentGet(shift, 'Registration'); }

# Alias for Accepted registration
sub DocumentGetAccepted
{ print DocumentGet(shift, 'Accepted'); }

# Alias for Javascript
sub DocumentGetJavascript
{ print DocumentGet(shift, 'WebJavascript'); }



#####################
# GetHash
#
# Sub-routine created to take a reference to an array and retrieve a list of
# all records within a specific field of information.

sub GetHash                                                     # Database handle, reference to array, SQL
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# SQL Statement

	my $hash     = shift;					# Reference of hash
	my $type     = shift;					# Type of values

	my $key;						# Key of hash
	my $value;						# Value for hash


	#####################
	# Program area

	# Prepare and execute
	$Statement = $Database->prepare("SELECT 
		 TagName, 
		 TagData
		FROM  Tag
		WHERE TagType=?");
	$Statement->execute($type);

	# Retreive and use
	while (($key, $value) = $Statement->fetchrow())
	{ $hash->{$key} = $value; }

	# Finish statement
	$Statement->finish();
}

# Alias for Chat hash
sub GetHashChat
{ GetHash(shift, shift, 'chat'); }

# Alias for Configuration hash
sub GetHashConfiguration
{ GetHash(shift, shift, 'configuration'); }

# Alias for Login hash
sub GetHashLogin
{ GetHash(shift, shift, 'login'); }

# Alias for Portal hash
sub GetHashPortal
{ GetHash(shift, shift, 'portal'); }

# Alias for Realm hash
sub GetHashRealm
{ GetHash(shift, shift, 'realm'); }

# Alias for System hash
sub GetHashSystem
{ GetHash(shift, shift, 'system'); }


#####################
# GetList
#
# Sub-routine created to take a reference to an array and retrieve a list of
# all records within a specific field of information.

sub GetList                                                     # Database handle, reference to array, SQL
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $array = shift;					# Reference of array
	my $query = shift;					# SQL query

	my $Statement;						# SQL Statement


	#####################
	# Program area

	# Prepare and execute
	$Statement = $Database->prepare($query);
	$Statement->execute();

	# Cleanup of array
	splice(@{ $array }, 0);

	# Retrieve and place in array
	while ($_ = $Statement->fetchrow())
	{ push(@{ $array }, $_); }

        # End statement
        $Statement->finish;
}

# Alias for documents
sub GetListDocuments
{ GetList(shift, shift, 'SELECT WebchatTextName FROM WebchatText ORDER BY WebchatTextName'); }

# Alias for public realms
sub GetListPublicRealms
{ GetList(shift, shift, "SELECT RealmName FROM Realm WHERE RealmPublic='yes' ORDER BY RealmName"); }

# Alias for private realms
sub GetListPrivateRealms
{ GetList(shift, shift, "SELECT RealmName FROM Realm WHERE RealmPublic='no' ORDER BY RealmName"); }

# Alias for private realms
sub GetListGlobalRealms
{ GetList(shift, shift, "SELECT RealmName FROM Realm ORDER BY RealmName"); }

# Alias for puppet listing other then puppeteers
sub GetListPuppetOther
{ GetList(shift, shift, "SELECT PuppetName FROM Puppet WHERE PuppeteerLogin <> '$_[0]' ORDER BY PuppetName"); }

# Alias for puppet listing that are yours
sub GetListPuppetYours
{ GetList(shift, shift, "SELECT PuppetName FROM Puppet WHERE PuppeteerLogin='$_[0]' AND PuppetActive IS NULL ORDER BY PuppetName"); }

# Alias for puppet listing that are yours
sub GetListPuppetIgnored
{ GetList(shift, shift, "SELECT PuppetName FROM PuppetIgnore WHERE PuppeteerLogin='$_[0]' ORDER BY PuppetName"); }

# Alias for puppet listing that are yours
sub GetListPuppetSuspended
{ GetList(shift, shift, "SELECT PuppetName FROM Puppet WHERE PuppeteerLogin='$_[0]' AND PuppetActive IS NOT NULL ORDER BY PuppetName"); }

# Alias for EMail retreival
sub GetListEMails
{ GetList(shift, shift, "SELECT DISTINCT(PuppeteerEMail) FROM Puppeteer ORDER BY PuppeteerEMail"); }

# Alias for who is online
sub GetListWho
{ GetList(shift, shift, "SELECT DISTINCT(PuppetName) FROM Post WHERE RealmName='$_[0]' AND PostTimestamp > '$_[1]' ORDER BY PuppetName"); }

# Alias for expired puppeteers
sub GetListExpire
{ GetList(shift, shift, "SELECT PuppeteerLogin FROM Puppeteer WHERE PuppeteerTimestamp < '$_[0]'"); }

# Alias for bans
sub GetListBanned
{ GetList(shift, shift, "SELECT PuppeteerLogin FROM PuppeteerRestriction WHERE RealmName='$_[0]'"); }

# Alias for restricted words
sub GetListWords
{ GetList(shift, shift, "SELECT WordSpelling FROM WordRestriction WHERE RealmName='$_[0]'"); }



#####################
# GetData
#
# Generic sub-routine that will return a single row of data be it in multiple
# amounts or just one field within the row there is a special section for
# font handling if choice is at all defined.

sub DataGet							# Database, query, choice
{
	#####################
	# Data members
	my $Database = shift;					# Database get
	my $Statement;						# SQL Statement
	my $Query = shift;					# Database query

	my $choice = shift;					# Choice of the matter

	my @values;						# Values returned from table


	#####################
	# Program area

	# Prepare and execute
	$Statement = $Database->prepare($Query);
	$Statement->execute();

	# Retreive and finish
	@values = $Statement->fetchrow();
	$Statement->finish();

	if (defined($choice))
	{
	       	# Verify and define
		$values[0] = (defined($values[0])) ? $values[0] : 3;
		$values[1] = (defined($values[1])) ? $values[1] : $choice;
	}

	# Return retreived information
	return @values;
}

# Alias for Supervisory option
sub DataGetAdmin
{ return DataGet(shift, "SELECT PuppeteerAdminPrivs FROM Puppeteer WHERE PuppeteerLogin='$_[0]'"); }

# Puppeteer
# Alias for Puppeteer Info
sub DataGetPuppeteerLogin
{ return DataGet(shift, "SELECT PuppeteerLogin FROM Puppet WHERE PuppetName='$_[0]'"); }

# Alias for font settings
sub DataGetFont
{ return DataGet(shift, "SELECT PuppetTextSize, PuppetTextColour FROM Puppet WHERE PuppetName='$_[0]'", $_[1]); }

# Alias for Javascript option
sub DataGetJavascript
{ return DataGet(shift, "SELECT PuppeteerJavascript FROM Puppeteer WHERE PuppeteerLogin='$_[0]'"); }

# Alias for chat options
sub DataGetChatOptions
{ return DataGet(shift, "SELECT PuppeteerJavascript, PuppeteerChatPref FROM Puppeteer WHERE PuppeteerLogin='$_[0]'"); }

# Alias for Formatting option
sub DataGetFormatting
{ return DataGet(shift, "SELECT PuppeteerFormatting FROM Puppeteer WHERE PuppeteerLogin='$_[0]'"); }

# Alias for Supervisory option
sub DataGetSupervisor
{ return DataGet(shift, "SELECT PuppeteerSuperPrivs FROM Puppeteer WHERE PuppeteerLogin='$_[0]'"); }

# Alias for Supervisory option
sub DataGetBannedStatus
{ return DataGet(shift, "SELECT PuppeteerLogin FROM PuppeteerRestriction WHERE PuppeteerLogin='$_[0]' AND RealmName='$_[1]'"); }

# Alias for PuppeteerEmail retreival
sub DataGetPuppeteerEmail
{ return DataGet(shift, "SELECT PuppeteerEmail FROM Puppeteer WHERE PuppeteerLogin='$_[0]'"); }


# Puppets
# Alias for Puppet Info
sub DataGetPuppetInfo
{ return DataGet(shift, "SELECT * FROM Puppet WHERE PuppetName='$_[0]'"); }

# Alias for Puppet Image Info
sub DataGetPuppetImageInfo
{ return DataGet(shift, "SELECT PuppetPicHeight, PuppetPicWidth FROM Puppet WHERE PuppetName='$_[0]'"); }

# Alias for Puppet Bio Info
sub DataGetPuppetBio
{ return DataGet(shift, "SELECT PuppetBio FROM Puppet WHERE PuppetName='$_[0]'"); }


# Realm
# Alias for Realm Description
sub DataGetRealmDescription
{ return DataGet(shift, "SELECT RealmInfo FROM Realm WHERE RealmName='$_[0]'"); }

# Alias for Realm Info
sub DataGetRealmInfo
{ return DataGet(shift, "SELECT RealmLanguage, RealmTags FROM Realm WHERE RealmName='$_[0]'"); }

# Alias for Realm Image Info
sub DataGetRealmImageInfo
{ return DataGet(shift, "SELECT RealmImageHeight, RealmImageWidth FROM Realm WHERE RealmName='$_[0]'"); }

# Alias for Complete Realm Info
sub DataGetRealmCompleteInfo
{ return DataGet(shift, "SELECT RealmName, RealmLanguage, RealmAccess, RealmGuests, RealmPublic, RealmTags, RealmInfo, RealmSidebar, RealmHeader, RealmFooter, RealmImageHeight, RealmImageWidth, RealmTimestamp FROM Realm WHERE RealmName='$_[0]'"); }

# Alias for Realm existence
sub DataGetRealmExistence
{ return DataGet(shift, "SELECT RealmName FROM Realm WHERE RealmName='$_[0]'"); }

# Alias for Realm HTML
sub DataGetRealmHtml
{ return DataGet(shift, "SELECT RealmFooter, RealmHeader FROM Realm WHERE RealmName='$_[0]'"); }

1;								# Return true
